@@ -5,7 +5,6 @@ from tempfile import mkdtemp
 from shutil import rmtree
 from textwrap import dedent
 
-import apt_pkg
 import imp
 
 from charmhelpers import osplatform
@@ -16,6 +15,7 @@ from tests.helpers import mock_open as mocked_open
 import six
 
 from charmhelpers.core import host
+from charmhelpers.fetch import ubuntu_apt_pkg
 
 
 MOUNT_LINES = ("""
@@ -69,7 +69,7 @@ IP_LINE_ETH1 = b"""
     link/ether e4:11:5b:ab:a7:3c brd ff:ff:ff:ff:ff:ff
 """
 
-IP_LINE_HWADDR = b"""2: eth0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast state UP qlen 1000\    link/ether e4:11:5b:ab:a7:3c brd ff:ff:ff:ff:ff:ff"""
+IP_LINE_HWADDR = b"""2: eth0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast state UP qlen 1000\\    link/ether e4:11:5b:ab:a7:3c brd ff:ff:ff:ff:ff:ff"""
 
 IP_LINES = IP_LINE_ETH0 + IP_LINE_ETH1 + IP_LINE_ETH0_VLAN + IP_LINE_ETH100
 
@@ -1128,6 +1128,56 @@ class HelpersTest(TestCase):
             os_.fchmod.assert_called_with(fileno, perms)
             mock_file.write.assert_called_with(b'what is {juju}')
 
+    @patch.object(host, 'log')
+    @patch.object(host, 'os')
+    def test_does_not_write_duplicate_content(self, os_, log):
+        uid = 0
+        gid = 0
+        path = '/some/path/{baz}'
+        fmtstr = b'what is {juju}'
+        perms = 0o444
+        fileno = 'some-fileno'
+
+        os_.stat.return_value.st_uid = 1
+        os_.stat.return_value.st_gid = 1
+        os_.stat.return_value.st_mode = 0o777
+
+        with patch_open() as (mock_open, mock_file):
+            mock_file.fileno.return_value = fileno
+            mock_file.read.return_value = fmtstr
+
+            host.write_file(path, fmtstr)
+
+            self.assertEqual(mock_open.call_count, 1)  # Called to read
+            os_.chown.assert_has_calls([
+                call(path, uid, -1),
+                call(path, -1, gid),
+            ])
+            os_.chmod.assert_called_with(path, perms)
+
+    @patch.object(host, 'log')
+    @patch.object(host, 'os')
+    def test_only_changes_incorrect_ownership(self, os_, log):
+        uid = 0
+        gid = 0
+        path = '/some/path/{baz}'
+        fmtstr = b'what is {juju}'
+        perms = 0o444
+        fileno = 'some-fileno'
+
+        os_.stat.return_value.st_uid = uid
+        os_.stat.return_value.st_gid = gid
+        os_.stat.return_value.st_mode = perms
+
+        with patch_open() as (mock_open, mock_file):
+            mock_file.fileno.return_value = fileno
+            mock_file.read.return_value = fmtstr
+
+            host.write_file(path, fmtstr)
+
+            self.assertEqual(mock_open.call_count, 1)  # Called to read
+            self.assertEqual(os_.chown.call_count, 0)
+
     @patch.object(host, 'log')
     @patch.object(host, 'os')
     def test_writes_binary_contents(self, os_, log):
@@ -1710,8 +1760,13 @@ class HelpersTest(TestCase):
         hwaddr = host.get_nic_hwaddr(nic)
         self.assertEqual(hwaddr, 'e4:11:5b:ab:a7:3c')
 
+    @patch('charmhelpers.core.host_factory.ubuntu.lsb_release')
+    def test_get_distrib_codename(self, lsb_release):
+        lsb_release.return_value = {'DISTRIB_CODENAME': 'bionic'}
+        self.assertEqual(host.get_distrib_codename(), 'bionic')
+
     @patch.object(osplatform, 'get_platform')
-    @patch.object(apt_pkg, 'Cache')
+    @patch.object(ubuntu_apt_pkg, 'Cache')
     def test_cmp_pkgrevno_revnos_ubuntu(self, pkg_cache, platform):
         platform.return_value = 'ubuntu'
         imp.reload(host)
@@ -1855,11 +1910,95 @@ class HelpersTest(TestCase):
         handle.write.assert_called_once_with(
             'PRUNEPATHS="/tmp /srv/node /tmp/test"')
 
+    @patch.object(host, 'os')
+    def test_prunepaths_no_updatedb_conf_file(self, mock_os):
+        mock_os.path.exists.return_value = False
+        _open = mock_open(read_data='PRUNEPATHS="/tmp /srv/node"')
+        with patch('charmhelpers.core.host.open', _open, create=True):
+            host.add_to_updatedb_prunepath("/tmp/test")
+        handle = _open()
+
+        self.assertTrue(handle.call_count == 0)
+
+    @patch.object(host, 'os')
+    def test_prunepaths_updatedb_conf_file_isdir(self, mock_os):
+        mock_os.path.exists.return_value = True
+        mock_os.path.isdir.return_value = True
+        _open = mock_open(read_data='PRUNEPATHS="/tmp /srv/node"')
+        with patch('charmhelpers.core.host.open', _open, create=True):
+            host.add_to_updatedb_prunepath("/tmp/test")
+        handle = _open()
+
+        self.assertTrue(handle.call_count == 0)
+
     @patch.object(host, 'local_unit')
     def test_modulo_distribution(self, local_unit):
         local_unit.return_value = 'test/7'
+
+        # unit % modulo * wait
         self.assertEqual(host.modulo_distribution(modulo=6, wait=10), 10)
 
+        # Zero wait when unit % modulo == 0
+        self.assertEqual(host.modulo_distribution(modulo=7, wait=10), 0)
+
+        # modulo * wait when unit % modulo == 0 and non_zero_wait=True
+        self.assertEqual(host.modulo_distribution(modulo=7, wait=10,
+                                                  non_zero_wait=True),
+                         70)
+
+    @patch.object(host, 'log')
+    @patch.object(host, 'charm_name')
+    @patch.object(host, 'write_file')
+    @patch.object(subprocess, 'check_call')
+    @patch.object(host, 'file_hash')
+    @patch('hashlib.md5')
+    def test_install_ca_cert_new_cert(self, md5, file_hash, check_call,
+                                      write_file, charm_name, log):
+        file_hash.return_value = 'old_hash'
+        charm_name.return_value = 'charm-name'
+
+        md5().hexdigest.return_value = 'old_hash'
+        host.install_ca_cert('cert_data')
+        assert not check_call.called
+
+        md5().hexdigest.return_value = 'new_hash'
+        host.install_ca_cert(None)
+        assert not check_call.called
+        host.install_ca_cert('')
+        assert not check_call.called
+
+        host.install_ca_cert('cert_data', 'name')
+        write_file.assert_called_with(
+            '/usr/local/share/ca-certificates/name.crt',
+            b'cert_data')
+        check_call.assert_called_with(['update-ca-certificates', '--fresh'])
+
+        host.install_ca_cert('cert_data')
+        write_file.assert_called_with(
+            '/usr/local/share/ca-certificates/juju-charm-name.crt',
+            b'cert_data')
+        check_call.assert_called_with(['update-ca-certificates', '--fresh'])
+
+    @patch('subprocess.check_output')
+    def test_arch(self, check_output):
+        _ = host.arch()
+        check_output.assert_called_with(
+            ['dpkg', '--print-architecture']
+        )
+
+    @patch('subprocess.check_output')
+    def test_get_system_env(self, check_output):
+        check_output.return_value = ''
+        self.assertEquals(
+            host.get_system_env('aKey', 'aDefault'), 'aDefault')
+        self.assertEquals(host.get_system_env('aKey'), None)
+        check_output.return_value = 'aKey=aValue\n'
+        self.assertEquals(
+            host.get_system_env('aKey', 'aDefault'), 'aValue')
+        check_output.return_value = 'otherKey=shell=wicked\n'
+        self.assertEquals(
+            host.get_system_env('otherKey', 'aDefault'), 'shell=wicked')
+
 
 class TestHostCompator(TestCase):
 
